import contextlib

from savory_pie.formatters import JSONFormatter

from mock import Mock


def mock_context():
    @contextlib.contextmanager
    def target(*args):
        ctx.push(*args)
        yield
        ctx.pop()

    ctx = Mock(name='context', spec=['push', 'pop', 'peek'])
    ctx.formatter = JSONFormatter()
    ctx.build_resource_uri = lambda resource: 'uri://' + resource.resource_path
    ctx.target = target
    return ctx



import unittest
import decimal
import datetime
import pytz

import savory_pie.formatters


class JSONToAPITest(unittest.TestCase):

    def setUp(self):
        self.json_formatter = savory_pie.formatters.JSONFormatter()
        self.now = datetime.datetime(2013, 3, 5, 14, 50, 39, 123456, pytz.UTC)
        self.json_now = '2013-03-05T14:50:39.123456+00:00'

        # date
        self.now_date = datetime.date(2013, 3, 5)
        self.json_now_date = '2013-03-05'

    def test_int(self):
        result = self.json_formatter.to_api_value(int, 15)
        self.assertEqual(15, result)

    def test_long(self):
        result = self.json_formatter.to_api_value(long, 15L)
        self.assertEqual(15L, result)

    def test_float(self):
        result = self.json_formatter.to_api_value(float, 15.5)
        self.assertEqual(15.5, result)

    def test_dict(self):
        result = self.json_formatter.to_api_value(dict, {'a': 1, 'b': 2})
        self.assertEqual({'a': 1, 'b': 2}, result)

    def test_list(self):
        result = self.json_formatter.to_api_value(list, [1, 2, 3])
        self.assertEqual([1, 2, 3], result)

    def test_bool(self):
        result = self.json_formatter.to_api_value(bool, True)
        self.assertEqual(True, result)

    def test_str(self):
        result = self.json_formatter.to_api_value(str, 'abc')
        self.assertEqual('abc', result)

    def test_unicode(self):
        result = self.json_formatter.to_api_value(unicode, u'abc\U0001F4A9')
        self.assertEqual(u'abc\U0001F4A9', result)

    def test_none(self):
        result = self.json_formatter.to_api_value(unicode, None)
        self.assertEqual(None, result)

    def test_decimal(self):
        result = self.json_formatter.to_api_value(decimal.Decimal, decimal.Decimal('5.10'))
        self.assertEqual('5.10', result)

    def test_datetime(self):
        result = self.json_formatter.to_api_value(datetime.datetime, self.now)
        self.assertEqual(self.json_now, result)

    def test_date(self):
        result = self.json_formatter.to_api_value(datetime.date, self.now_date)
        self.assertEqual(self.json_now_date, result)

    def test_empty_datetime(self):
        result = self.json_formatter.to_api_value(datetime.datetime, None)
        self.assertEqual(None, result)

    def test_empty_date(self):
        result = self.json_formatter.to_api_value(datetime.date, None)
        self.assertEqual(None, result)


class JSONToPython(unittest.TestCase):

    def setUp(self):
        self.json_formatter = savory_pie.formatters.JSONFormatter()
        self.now = datetime.datetime(2013, 3, 5, 14, 50, 39, 123456, pytz.UTC)
        self.json_now = '2013-03-05T14:50:39.123456+00:00'
        self.json_now_alternative = '2013-03-05T14:50:39.123456Z'

        # date
        self.now_date = datetime.date(2013, 3, 5)
        self.json_now_date = '2013-03-05'

    def test_int(self):
        result = self.json_formatter.to_python_value(int, 15)
        self.assertEqual(15, result)

    def test_long(self):
        result = self.json_formatter.to_python_value(long, 15)
        self.assertEqual(15L, result)

    def test_float(self):
        result = self.json_formatter.to_python_value(float, 15.5)
        self.assertEqual(15.5, result)

    def test_dict(self):
        result = self.json_formatter.to_python_value(dict, {'a': 1, 'b': 2})
        self.assertEqual({'a': 1, 'b': 2}, result)

    def test_list(self):
        result = self.json_formatter.to_python_value(list, [1, 2, 3])
        self.assertEqual([1, 2, 3], result)

    def test_bool(self):
        result = self.json_formatter.to_python_value(bool, True)
        self.assertEqual(True, result)

    def test_str(self):
        result = self.json_formatter.to_python_value(str, 'abc')
        self.assertEqual('abc', result)

    def test_unicode(self):
        result = self.json_formatter.to_python_value(unicode, u'abc\U0001F4A9')
        self.assertEqual(u'abc\U0001F4A9', result)

    def test_none(self):
        result = self.json_formatter.to_python_value(unicode, None)
        self.assertEqual(None, result)

    def test_decimal(self):
        result = self.json_formatter.to_python_value(decimal.Decimal, '5.10')
        self.assertEqual(decimal.Decimal('5.10'), result)

    def test_more_decimal(self):
        for x in ['15.5', '1.550000e1', '0.000155e5', '1550e-2',
                  '1550000000000000000.0e-017',
                  '0.000000000000000155e+017']:
            result = self.json_formatter.to_python_value(decimal.Decimal, x)
            self.assertEqual(15.5, result)

    def test_datetime(self):
        result = self.json_formatter.to_python_value(datetime.datetime, self.json_now)
        self.assertEqual(self.now, result)

    def test_datetime_alternative(self):
        result = self.json_formatter.to_python_value(datetime.datetime, self.json_now_alternative)
        self.assertEqual(self.now, result)

    def test_none_datetime(self):
        result = self.json_formatter.to_python_value(datetime.datetime, None)
        self.assertEqual(None, result)

    def test_crazy_datetimes(self):
        for craziness in ('', False, {}, [], 3.14159, 3 + 4j,
                          'Rumplestiltskin', self, (3 + 4j).conjugate):
            try:
                self.json_formatter.to_python_value(datetime.datetime, craziness)
                self.fail(repr(craziness) + ' should not be parsable as a datetime')
            except TypeError:
                pass

    def test_date(self):
        result = self.json_formatter.to_python_value(datetime.date, self.json_now_date)
        self.assertEqual(self.now_date, result)

    def test_none_date(self):
        result = self.json_formatter.to_python_value(datetime.date, None)
        self.assertEqual(None, result)

    def test_unparsable_data(self):
        # should raise a TypeError
        # The following datatypes can parse any string as legal data so we don't try
        # to test them here: list, bool, str, unicode.
        unparsable = 'unparsable_data'
        for _type, svalue in [(int, unparsable),
                              (long, unparsable),
                              (float, unparsable),
                              (dict, unparsable),
                              (type(None), unparsable),
                              (datetime.datetime, unparsable)]:
            message = ('Expected a TypeError for unparsable ' + str(_type) +
                       ' with value ' + repr(svalue))
            succeeded_incorrectly = False
            try:
                self.json_formatter.to_python_value(_type, svalue)
                succeeded_incorrectly = True
            except TypeError:
                pass
            except Exception, e:
                self.fail(message + ', got ' + str(e.__class__))
            if succeeded_incorrectly:
                self.fail(message)



from StringIO import StringIO

from savory_pie.django import views
from savory_pie.tests.mock_context import mock_context as _mock_context


def savory_dispatch(root_resource, method, resource_path='', body=None, GET=None, POST=None):
    view = views.api_view(root_resource)
    request = Request(
        method=method,
        resource_path=resource_path,
        body=body,
        GET=GET,
        POST=POST
    )

    return view(request=request, resource_path=resource_path)


def savory_dispatch_batch(
        root_resource, method,
        resource_path='',
        full_host='',
        body=None,
        GET=None,
        POST=None,
        base_regex=None
):
    view = views.batch_api_view(root_resource, base_regex)
    request = Request(
        method=method,
        host=full_host,
        resource_path=resource_path,
        body=body,
        GET=GET,
        POST=POST
    )

    return view(request=request, resource_path=resource_path)


class User(object):
    email = 'savory-pie-tester@localhost'

    def is_authenticated(self):
        return True


class Request(object):
    def __init__(self, method, host='localhost', resource_path='', body=None, GET=None, POST=None):
        self.host = host
        self.resource_path = resource_path

        self.user = User()

        self.method = method
        self.body = body
        self.body_file = None

        self.GET = GET or {}
        self.POST = POST or {}
        self.META = {}
        self.REQUEST = dict(self.GET, **self.POST)

    def get_host(self):
        return self.host

    def get_full_path(self):
        return 'api/' + self.resource_path

    def build_absolute_uri(self, django_path):
        return 'http://' + self.host + '/' + django_path

    def read(self):
        if not self.body_file:
            self.body_file = StringIO(self.body)

        return self.body_file.read()


def mock_context(*args, **kwargs):
    ctx = _mock_context(*args, **kwargs)
    setattr(ctx, 'request', Request('None'))
    return ctx



#protocol Params:
#   def get(key, default=None)
#   def get_as(key, type, default=None)
#   def get_list(key)
#   def get_list_of(key, type)


class EmptyParams(object):
    def get(self, key, default=None):
        return default

    def get_as(self, key, type, default=None):
        return default

    def get_list(self, key):
        return []

    def get_list_of(self, key, type):
        return []

    def keys(self):
        return []


class Resource(object):
    """
    Base object for defining resources.
    """
    #: Internal path (from root of the resource tree to this Resource).  If not
    #: set, this is auto-filled during Resource traversal; however, if you wish
    #: for a Resource to always be addressable, resource_path should be set at
    #: construction.
    resource_path = None

    validators = []

    @property
    def allowed_methods(self):
        """
        defaults to set of available methods based on
        presence of the optional methods - get, post, put, etc.

        Can be overridden with a static set or dynamic property to
        create access controls.
        """
        allowed_methods = set()

        for http_method in ['GET', 'POST', 'PUT', 'DELETE']:
            obj_method = http_method.lower()
            try:
                getattr(self, obj_method)
                allowed_methods.add(http_method)
            except AttributeError:
                pass

        return allowed_methods

    # def get(self, ctx, params):
        """
        Optional method that is called during a GET request.

        get is provided an APIContext and an optional set of kwargs that include the
        query string params.

        Returns a dict of data to be serialized to the requested format.
        """

    # def post(self, ctx, dict):
        """
        Optional method that is called during a POST request.

        post is provided with a dict representing the deserialized representation of
        the body content.

        Returns a new Resource
        """

    # def put(self, ctx, dict, save):
        """
        Optional method that is called during a PUT request.

        put is provided with a dict representing the deserialized representation of
        the body content.
        """

    # def delete(self, ctx):
        """
        Optional method that is called during a DELETE request.
        """

    def get_child_resource(self, ctx, path_fragment):
        return None


class APIResource(Resource):
    def __init__(self, resource_path=''):
        self.resource_path = resource_path
        self._child_resources = dict()
        self._base_regex = ''

    def set_base_regex(self, base_path):
        self._base_regex = base_path

    @property
    def base_regex(self):
        return self._base_regex

    def register(self, resource):
        """
        Register a resource into the API.
        """
        leaf = resource.resource_path
        if '/' in resource.resource_path:
            n = resource.resource_path.index('/')
            leaf = resource.resource_path[n + 1:]

        self._child_resources[leaf] = resource
        return self

    def register_class(self, resource_class):
        """
        Register a resource class into the API.  The constructed Resource
        must have a first-level resource_path set after construction.
        """
        return self.register(resource_class())

    def get_child_resource(self, ctx, path_fragment):
        return self._child_resources.get(path_fragment, None)


class _ParamsImpl(object):
    def __init__(self, GET):
        self._GET = GET

    def keys(self):
        return self._GET.keys()

    def __contains__(self, key):
        return key in self._GET

    def __getitem__(self, key):
        return self._GET.get(key, None)

    def get(self, key, default=None):
        return self._GET.get(key, default)

    def get_as(self, key, type, default=None):
        value = self._GET.get(key, None)
        return default if value is None else type(value)

    def get_list(self, key):
        return self._GET.getlist(key)

    def get_list_of(self, key, type):
        list = self._GET.get(key, None)
        if list is None:
            return []
        else:
            return [type(x) for x in list]



from haystack.query import SearchQuerySet

from savory_pie.django.filters import ParameterizedFilter


class HaystackFilter(ParameterizedFilter):
    """
    Assuming Haystack is available as a way to search models, use a Haystack search instead
    of a Django filter operation. The filter parameter is treated as a Haystack search term.
    Any additional search criteria, or the order_by stuff, is subsequently treated normally.
    """

    def __init__(self, name='q', criteria=None, order_by=None):
        ParameterizedFilter.__init__(self, name, name, criteria, order_by)

    def filter(self, ctx, params, queryset):
        name = self.is_applicable(ctx, params)
        if name:
            for value in self.get_param_values(name, ctx, params):
                value = unicode(value)
                hs_results = SearchQuerySet().models(queryset.model)
                for word in value.split():
                    hs_results = hs_results.filter(content=word)
                pks = [result.pk for result in hs_results]
                queryset = queryset.filter(pk__in=pks)
                queryset = self.apply(self.criteria, queryset)
        return queryset



import functools
import re

try:
    import newrelic.agent as agent

    def set_transaction_name(func):
        @functools.wraps(func)
        def inner(request, resource_path):
            agent.set_transaction_name(
                get_transaction_name_path(resource_path),
                'Python/WebFramework/Controller'
            )
            return func(request, resource_path)
        return inner

except ImportError:
    def set_transaction_name(func):
        return func


def get_transaction_name_path(resource_path):
    """
    Takes a resource path and removes its last fragment if and only if that fragment is just digits
    e.g.
        'bars/foo' => 'bars/foo'
        'bars/bar/9' => 'bars/bar'
        'bars/19229' => 'bars'
        'bars/magic8ball' => 'bars/magic8ball'
    """
    path_fragments = resource_path.split('/')

    if re.match(r'^\d+$', path_fragments[-1]):
        return '/'.join(path_fragments[:-1])
    else:
        return resource_path



import exceptions
try:
    import ujson as json
except ImportError:
    from warnings import warn
    warn('Using plain JSON instead of uJSON, performance may be degraded.')
    import json
import pytz
import datetime
import re

from dateutil import parser


class JSONFormatter(object):
    """
    Formatter reads and writes json while converting properties to and from
    javascript naming conventions and pep8.
    """

    content_type = 'application/json'

    dateRegex = re.compile('(\d{4})-(\d{2})-(\d{2})(.*)')

    def parse_datetime(self, s):
        if s is None:
            return None
        if self.dateRegex.match(s):
            try:
                return parser.parse(s).astimezone(pytz.utc)
            except ValueError:
                return parser.parse(s).replace(tzinfo=pytz.utc)
        raise TypeError('Unable to parse ' + repr(s) + ' as a datetime')

    def parse_date(self, s):
        if s is None:
            return None
        if self.dateRegex.match(s):
            return parser.parse(s).date()
        raise TypeError('Unable to parse ' + repr(s) + ' as a datetime')

    def convert_to_public_property(self, bare_attribute):
        parts = bare_attribute.split('_')
        return ''.join([parts[0], ''.join(x.capitalize() for x in parts[1:])])

    def read_from(self, request):
        return json.load(request)

    def write_to(self, body_dict, response):
        json.dump(body_dict, response)

    # Not 100% happy with this API review pre 1.0
    def to_python_value(self, type_, api_value):
        try:
            if type_ is datetime.date:
                return self.parse_date(api_value)
            elif issubclass(type_, datetime.datetime):
                return self.parse_datetime(api_value)
            return None if api_value is None else type_(api_value)
        except (ValueError, exceptions.StandardError):
            raise TypeError('Expected ' + str(type_) + ', got ' + repr(api_value))

    # Not 100% happy with this API review pre 1.0
    def to_api_value(self, type_, python_value):
        if python_value is not None:
            if type_ is datetime.date:
                return python_value.strftime("%Y-%m-%d")
            elif issubclass(type_, datetime.datetime):
                #Check if it is a naive date, and if so, make it UTC
                if not python_value.tzinfo:
                    python_value = python_value.replace(tzinfo=pytz.UTC)
                return python_value.isoformat("T")
            elif type(python_value) not in (int, long, float, dict, list,
                                            bool, str, unicode, type(None)):
                return str(python_value)

        return python_value



import os

# Only here so django can be imported
USE_TZ = True
SECRET_KEY = 'ecret-say_ey-kay'

PROJECT_ROOT = os.path.join(os.path.dirname(__file__))
TEMPLATE_DIRS = (os.path.join(PROJECT_ROOT, 'templates'),)

HAYSTACK_CONNECTIONS = {
    'default': {
        'ENGINE': 'haystack.backends.simple_backend.SimpleEngine',
    },
}

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = [
    'haystack',
]

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
    },
    'loggers': {
        'savory_pie': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'filters': []
        }
    }
}



try:
    import ujson as json
except ImportError:
    from warnings import warn
    warn('Using plain JSON instead of uJSON, performance may be degraded.')
    import json
import logging

from haystack import indexes
from haystack import fields as haystack_fields

from savory_pie.context import APIContext
from savory_pie.django.utils import Related


logger = logging.getLogger(__name__)


class ResourceIndex(indexes.SearchIndex):

    def prefetch_related(self, related):
        pass

    def get_model(self):
        return self.resource_class.model_class

    def _prefetch_related(self, qs):
        related = Related()
        ctx = APIContext('', None, None)
        self.resource_class.prepare(ctx, related)
        self.prefetch_related(related)
        return related.prepare(qs)

    def index_queryset(self, using=None):
        qs = self.get_model().objects.all()
        return self._prefetch_related(qs)

    def build_queryset(self, start_date=None, end_date=None, using=None):
        return self.index_queryset(using=using)


class HaystackField(haystack_fields.CharField):
    """
    This field can be used to store the JSON from an API call into a Haystack search database.
    It typically wouldn't be indexed (but could be if you wanted). Typical usage:

        from haystack import indexes, fields
        from savory_pie.django.fields import HaystackField

        class FooIndex(indexes.SearchIndex, indexes.Indexable):
            foo = fields.CharField(...)
            bar = fields.CharField(...)
            api = HaystackField(base_uri='/my/api/path/',
                                formatter=JSONFormatter(),
                                resource=FooResource)
    """
    def __init__(self, *args, **kwargs):
        self._formatter = kwargs.pop('formatter', None)
        self._ctx = APIContext('SAVORY_PIE_HOSTNAME', None, self._formatter)
        self._resource = kwargs.pop('resource', None)
        self.indexed = kwargs.pop('indexed', False)
        self.stored = kwargs.pop('indexed', True)
        super(HaystackField, self).__init__(*args, **kwargs)

    def prepare(self, obj):
        try:
            import cStringIO as StringIO
        except ImportError:
            import StringIO
        api_data = self._resource(obj).get(self._ctx, {})

        if self._formatter is None:
            return json.dumps(api_data)
        else:
            output = StringIO.StringIO()
            self._formatter.write_to(api_data, output)
            return output.getvalue()



import unittest
try:
    import ujson as json
except ImportError:
    from warnings import warn
    warn('Using plain JSON instead of uJSON, performance may be degraded.')
    import json
from datetime import datetime

import mock
from mock import Mock, patch
from savory_pie.errors import AuthorizationError, PreConditionError
from savory_pie.formatters import JSONFormatter
from savory_pie.resources import _ParamsImpl
from savory_pie.helpers import get_sha1
from savory_pie.django import validators
from savory_pie.tests.django.mock_request import savory_dispatch, savory_dispatch_batch
from savory_pie.tests.mock_context import mock_context


def mock_resource(name=None, resource_path=None, child_resource=None, base_regex=None):
    resource = Mock(name=name, spec=['set_base_regex'])
    resource.resource_path = resource_path

    resource.allowed_methods = set()
    resource.get = Mock(name='get')
    resource.post = Mock(name='post')
    resource.put = Mock(name='put')
    resource.delete = Mock(name='delete')
    resource.base_regex = base_regex

    resource.get_child_resource = Mock(return_value=child_resource)

    return resource


def call_args_sans_context(mock):
    return list(mock.call_args[0][1:])


class BatchViewTest(unittest.TestCase):

    def _generate_batch_partial(self, method, uri, body):
        return {
            "method": method, "uri": uri, "body": body
        }

    def create_root_resource_with_children(self, base_regex, methods=frozenset(), result=frozenset()):
        grand_child_resource = mock_resource(name='grandchild')
        child_resource = mock_resource(name='child', child_resource=grand_child_resource)

        for method in methods:
            grand_child_resource.allowed_methods.add(method)
            child_resource.allowed_methods.add(method)

        child_resource.get = Mock(return_value=result)
        child_resource.put = Mock(return_value=result)
        child_resource.post = Mock(return_value=result)

        grand_child_resource.get = Mock(return_value=result)
        grand_child_resource.put = Mock(return_value=result)
        grand_child_resource.post = Mock(return_value=result)

        root_resource = mock_resource(name='root', child_resource=child_resource, base_regex=base_regex)

        return root_resource

    def test_no_post_root_resource(self):
        root_resource = self.create_root_resource_with_children(
            r'^api/v2/(?P<base_resource>.*)$',
            methods=['GET'],
            result={'name': 'value'}
        )
        request_data = {
            "data": [{"method": "get", "uri": "someurl", "body": {}}]
        }
        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='GET',
            body=json.dumps(request_data)
        )

        self.assertEqual(response.status_code, 405)

    def test_put_nocontent_batch(self):
        root_resource = self.create_root_resource_with_children(
            r'^api/v2/(?P<base_resource>.*)$',
            methods=['PUT'],
            result={}
        )
        request_data = {
            "data": [
                self._generate_batch_partial(
                    'put',
                    'http://localhost:8081/api/v2/child/grandchild',
                    {'business_id': 12345}
                )
            ]
        }
        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=json.dumps(request_data)
        )
        self.assertEqual(response.status_code, 200)

        response_json = json.loads(response.content)
        data = response_json['data']

        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['status'], 204)
        self.assertEqual(data[0]['uri'], 'http://localhost:8081/api/v2/child/grandchild')
        self.assertTrue('data' not in data[0])

    def test_put_batch(self):
        root_resource = self.create_root_resource_with_children(
            r'^api/v2/(?P<base_resource>.*)$',
            methods=['PUT'],
            result={'name': 'value'}
        )
        request_data = {
            "data": [
                self._generate_batch_partial(
                    'put',
                    'http://localhost:8081/api/v2/child/grandchild',
                    {'business_id': 12345}
                )
            ]
        }
        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=json.dumps(request_data)
        )
        self.assertEqual(response.status_code, 200)

        response_json = json.loads(response.content)
        data = response_json['data']

        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['status'], 200)
        self.assertEqual(data[0]['uri'], 'http://localhost:8081/api/v2/child/grandchild')
        self.assertEqual(data[0]['data'], {u'name': u'value'})

    def test_put_precondition_batch(self):
        root_resource = mock_resource(
            name='root',
            base_regex=r'^api/v2/(?P<base_resource>.*)$'
        )

        root_resource.allowed_methods.add('PUT')

        root_resource.get.side_effect = PreConditionError

        request_data = {
            "data": [
                self._generate_batch_partial(
                    'put',
                    'http://localhost:8081/api/v2/',
                    {'business_id': 12345}
                )
            ]
        }
        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=json.dumps(request_data)
        )
        self.assertEqual(response.status_code, 200)

        data = json.loads(response.content)['data']
        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['status'], 412)

    def test_put_key_error_batch(self):
        root_resource = mock_resource(
            name='root',
            base_regex=r'^api/v2/(?P<base_resource>.*)$'
        )

        root_resource.allowed_methods.add('PUT')

        root_resource.get.side_effect = KeyError('bad key message')

        request_data = {
            "data": [
                self._generate_batch_partial(
                    'put',
                    'http://localhost:8081/api/v2/',
                    {'business_id': 12345}
                )
            ]
        }
        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=json.dumps(request_data)
        )
        self.assertEqual(response.status_code, 200)

        data = json.loads(response.content)['data']
        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['status'], 400)
        self.assertEqual(data[0]['validation_errors'], {'missingData': 'bad key message'})

    def test_get_batch(self):
        root_resource = self.create_root_resource_with_children(
            r'^api/v2/(?P<base_resource>.*)$',
            methods=['GET'],
            result={'name': 'value'}
        )
        request_data = {
            "data": [
                self._generate_batch_partial(
                    'get',
                    'http://localhost:8081/api/v2/child/grandchild',
                    {'business_id': 12345}
                )
            ]
        }
        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=json.dumps(request_data)
        )
        self.assertEqual(response.status_code, 200)

        response_json = json.loads(response.content)
        data = response_json['data']

        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['status'], 200)
        self.assertEqual(data[0]['uri'], 'http://localhost:8081/api/v2/child/grandchild')
        self.assertEqual(data[0]['data'], {u'name': u'value'})

        ctx = mock_context()
        ctx.formatter = JSONFormatter()

        self.assertEqual(data[0]['etag'], get_sha1(ctx, {u'name': u'value'}))

    def test_post_batch(self):
        result = Mock(resource_path='grand_child_path')
        root_resource = self.create_root_resource_with_children(
            r'^api/v2/(?P<base_resource>.*)$',
            methods=['POST'],
            result=result
        )
        request_data = {
            "data": [
                self._generate_batch_partial('post', 'http://localhost:8081/api/v2/child/grandchild', {})
            ]
        }
        body = json.dumps(request_data)
        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=body
        )

        self.assertEqual(response.status_code, 200)

        response_json = json.loads(response.content)

        data = response_json['data']

        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['status'], 201)
        self.assertEqual(data[0]['uri'], 'http://localhost:8081/api/v2/child/grandchild')
        self.assertEqual(data[0]['location'], 'http://localhost:8081/api/v2/grand_child_path')

    @patch('traceback.format_exc')
    def test_exception_in_batch(self, format_exc):
        format_exc.return_value = 'some traceback'
        root_resource = mock_resource(
            name='root',
            base_regex=r'^api/v2/(?P<base_resource>.*)$'
        )

        root_resource.allowed_methods.add('GET')
        root_resource.allowed_methods.add('POST')

        root_resource.get.side_effect = Exception

        request_data = {
            "data": [
                self._generate_batch_partial('get', 'http://localhost:8081/api/v2/', {'business_id': 12345})
            ]
        }
        body = json.dumps(request_data)
        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=body
        )
        self.assertEqual(response.status_code, 200)

        data = json.loads(response.content)['data']
        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['status'], 500)
        self.assertEqual(data[0]['data'], {'error': 'some traceback'})

    def test_unauthorized_batch(self):
        child_resource = mock_resource(name='child')
        root_resource = mock_resource(
            name='root',
            child_resource=child_resource,
            base_regex=r'^api/v2/(?P<base_resource>.*)$'
        )
        child_resource.allowed_methods.add('GET')
        child_resource.get.side_effect = AuthorizationError('foo')

        request_data = {
            "data": [
                self._generate_batch_partial('get', 'http://localhost:8081/api/v2/child', {'business_id': 12345})
            ]
        }
        body = json.dumps(request_data)
        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=body
        )
        self.assertEqual(response.status_code, 200)

        data = json.loads(response.content)['data']

        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['status'], 403)
        self.assertEqual(data[0]['validation_errors'], [u'Modification of field foo not authorized'])

    def test_method_not_allowed(self):
        root_resource = mock_resource(
            name='root',
            base_regex=r'^api/v2/(?P<base_resource>.*)$'
        )

        root_resource.allowed_methods.add('POST')

        root_resource.get.side_effect = Exception

        request_data = {
            "data": [
                self._generate_batch_partial('get', 'http://localhost:8081/api/v2/', {'business_id': 12345})
            ]
        }
        body = json.dumps(request_data)
        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=body
        )
        self.assertEqual(response.status_code, 200)

        data = json.loads(response.content)['data']
        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['status'], 405)
        self.assertEqual(data[0]['allowed'], 'POST')

    def test_validation_exception_batch(self):
        root_resource = mock_resource(
            name='root',
            base_regex=r'^api/v2/(?P<base_resource>.*)$'
        )

        root_resource.allowed_methods.add('POST')

        root_resource.post.side_effect = validators.ValidationError(
            Mock(),
            {
                'class.field': 'broken',
            }
        )

        request_data = {
            "data": [
                self._generate_batch_partial(
                    'post',
                    'http://localhost:8081/api/v2/',
                    {'business_id': 12345}
                )
            ]
        }

        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=json.dumps(request_data)
        )
        self.assertEqual(response.status_code, 200)

        data = json.loads(response.content)['data']
        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['status'], 400)
        self.assertEqual(data[0]['validation_errors'], {'class.field': 'broken'})

    def test_post_with_collision_two_batch(self):
        def side_effect(*args, **kwargs):
            # This could occur if a slightly earlier POST or PUT still had
            # the database locked during a DB transaction.
            from django.db.transaction import TransactionManagementError

            raise TransactionManagementError()

        child_resource = mock_resource(name='child')
        root_resource = mock_resource(
            name='root',
            child_resource=child_resource,
            base_regex=r'^api/v2/(?P<base_resource>.*)$'
        )
        child_resource.allowed_methods.add('POST')
        child_resource.get.side_effect = AuthorizationError('foo')
        child_resource.post = side_effect

        request_data = {
            "data": [
                self._generate_batch_partial('post', 'http://localhost:8081/api/v2/child', {'id ': 12345})
            ]
        }

        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=json.dumps(request_data)
        )

        data = json.loads(response.content)['data']
        self.assertEqual(len(data), 1)

        self.assertEqual(data[0]['status'], 409)
        self.assertEqual(data[0]['uri'], 'http://localhost:8081/api/v2/child')

    @patch('savory_pie.django.views.APIContext.build_resource_uri')
    def test_post_get_put(self, build_resource_uri):
        build_resource_uri.return_value = 'some new location'
        root_resource = self.create_root_resource_with_children(
            r'^api/v2/(?P<base_resource>.*)$',
            methods=['PUT', 'GET', 'POST'],
            result={'name': 'value'}
        )

        request_data = {
            "data": [
                self._generate_batch_partial(
                    'put',
                    'http://localhost:8081/api/v2/child/grandchild',
                    {'business_id': 12345}
                ), self._generate_batch_partial(
                    'get',
                    'http://localhost:8081/api/v2/child',
                    {'business_id': 12345}
                ), self._generate_batch_partial(
                    'post',
                    'http://localhost:8081/api/v2/child/grandchild',
                    {'business_id': 12345}
                )
            ]
        }

        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=json.dumps(request_data)
        )
        self.assertEqual(response.status_code, 200)

        response_json = json.loads(response.content)
        data = response_json['data']
        self.assertEqual(len(data), 3)
        self.assertEqual(data[0]['status'], 200)
        self.assertEqual(data[0]['uri'], 'http://localhost:8081/api/v2/child/grandchild')
        self.assertEqual(data[0]['data'], {u'name': u'value'})

        self.assertEqual(data[1]['status'], 200)
        self.assertEqual(data[1]['uri'], 'http://localhost:8081/api/v2/child')
        self.assertEqual(data[1]['data'], {u'name': u'value'})
        ctx = mock_context()
        ctx.formatter = JSONFormatter()

        self.assertEqual(data[1]['etag'], get_sha1(ctx, {u'name': u'value'}))

        self.assertEqual(data[2]['status'], 201)
        self.assertEqual(data[2]['uri'], 'http://localhost:8081/api/v2/child/grandchild')
        self.assertEqual(data[2]['location'], 'some new location')

    def test_one_fails_one_passes(self):
        root_resource = self.create_root_resource_with_children(
            r'^api/v2/(?P<base_resource>.*)$',
            methods=['PUT'],
            result={'name': 'value'}
        )

        request_data = {
            "data": [
                self._generate_batch_partial(
                    'put',
                    'http://localhost:8081/api/v2/child/grandchild',
                    {'business_id': 12345}
                ), self._generate_batch_partial(
                    'get',
                    'http://localhost:8081/api/v2/child',
                    {'business_id': 12345}
                )
            ]
        }

        response = savory_dispatch_batch(
            root_resource,
            full_host='localhost:8081',
            method='POST',
            body=json.dumps(request_data)
        )
        self.assertEqual(response.status_code, 200)

        response_json = json.loads(response.content)
        data = response_json['data']

        self.assertEqual(len(data), 2)
        self.assertEqual(data[0]['status'], 200)
        self.assertEqual(data[1]['status'], 405)


class ViewTest(unittest.TestCase):
    def test_unauthorized(self):
        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('PUT')
        root_resource.put.side_effect = AuthorizationError('foo')

        response = savory_dispatch(root_resource, method='PUT', body='{"foo": "bar"}')
        response_json = json.loads(response.content)

        self.assertIn('validation_errors', response_json)
        self.assertEqual(response_json['validation_errors'], ['Modification of field foo not authorized'])
        self.assertEqual(response.status_code, 403)

    def test_get_success(self):
        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('GET')
        root_resource.get = Mock(return_value={'foo': 'bar'})

        response = savory_dispatch(root_resource, method='GET')

        self.assertEqual(response.content, '{"foo": "bar"}')
        self.assertTrue(root_resource.get.called)
        self.assertIsNotNone(root_resource.get.call_args_list[0].request)

    def test_get_success_streaming(self):
        def get(ctx, params):
            ctx.streaming_response = True
            ctx.formatter = Mock()
            ctx.formatter.write_to = lambda *args: None
            return iter([
                '{"foo": ',
                '"bar"',
                '}',
            ])

        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('GET')
        root_resource.get = Mock(side_effect=get)

        response = savory_dispatch(root_resource, method='GET')

        self.assertEqual(response.status_code, 200)
        self.assertEqual(''.join(response.streaming_content), '{"foo": "bar"}')
        self.assertTrue(root_resource.get.called)
        self.assertIsNotNone(root_resource.get.call_args_list[0].request)

    def test_get_not_supported(self):
        root_resource = mock_resource(name='root')

        response = savory_dispatch(root_resource, method='GET')
        self.assertEqual(response.status_code, 405)

    def test_put_no_content_success(self):
        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('PUT')
        root_resource.get.return_value = {}
        root_resource.put.return_value = None

        response = savory_dispatch(root_resource, method='PUT', body='{"foo": "bar"}')

        self.assertTrue(
            call_args_sans_context(root_resource.put),
            [
                {
                    'foo': 'bar'
                }
            ]
        )

        self.assertEqual(response.status_code, 204)
        self.assertEqual(response.content, '')
        self.assertIsNotNone(root_resource.put.call_args_list[0].request)

    def test_put_content_success(self):
        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('PUT')
        root_resource.get.return_value = {}
        root_resource.put.return_value = {'key': 'value'}

        response = savory_dispatch(root_resource, method='PUT', body='{"foo": "bar"}')

        self.assertTrue(
            call_args_sans_context(root_resource.put),
            [
                {
                    'foo': 'bar'
                }
            ]
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, '{"key": "value"}')
        self.assertIsNotNone(root_resource.put.call_args_list[0].request)

    def test_put_not_supported(self):
        root_resource = mock_resource(name='root')

        response = savory_dispatch(root_resource, method='PUT', body='{}')
        self.assertEqual(response.status_code, 405)

    def test_post_success(self):
        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('POST')

        new_resource = mock_resource(name='new', resource_path='foo')
        root_resource.post = Mock(return_value=new_resource)

        response = savory_dispatch(root_resource, method='POST', body='{}')

        self.assertTrue(
            call_args_sans_context(root_resource.post),
            [
                {
                    'foo': 'bar'
                }
            ]
        )
        self.assertEqual(response['Location'], 'http://localhost/api/foo')
        self.assertIsNotNone(root_resource.post.call_args_list[0].request)

    def test_post_with_collision_one(self):
        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('POST')

        def side_effect(*args):
            # This could occur if a slightly earlier POST or PUT still had
            # the database locked during a DB transaction.
            from django.db.transaction import TransactionManagementError

            raise TransactionManagementError()

        root_resource.post = Mock(side_effect=side_effect)

        response = savory_dispatch(root_resource, method='POST', body='{}')
        self.assertEqual(response.status_code, 409)
        self.assertEqual(response.content, '{"resource": "http://localhost/api/"}')

    def test_post_with_collision_two(self):
        def side_effect(*args, **kwargs):
            # This could occur if a slightly earlier POST or PUT still had
            # the database locked during a DB transaction.
            from django.db.transaction import TransactionManagementError

            raise TransactionManagementError()

        root_resource = mock_resource(name='root')
        new_resource = mock_resource(name='new', resource_path='foo')
        root_resource.post = Mock(return_value=new_resource)
        root_resource.post.side_effect = side_effect
        root_resource.allowed_methods.add('POST')

        response = savory_dispatch(root_resource, method='POST', body='{}')
        self.assertEqual(response.status_code, 409)
        self.assertEqual(response.content, '{"resource": "http://localhost/api/"}')

    @mock.patch('savory_pie.django.views.logger')
    def test_post_with_exception(self, logger):
        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('POST')

        def side_effect(*args):
            raise Exception('Some kind of server error')

        root_resource.post = Mock(side_effect=side_effect)

        response = savory_dispatch(root_resource, method='POST', body='{}')
        self.assertEqual(response.status_code, 500)
        content = json.loads(response.content)
        self.assertTrue('error' in content)
        self.assertTrue(content['error'].startswith('Traceback (most recent call last):'))
        self.assertTrue('Some kind of server error' in content['error'])
        self.assertTrue(logger.exception.called)

    def test_post_not_supported(self):
        root_resource = mock_resource(name='root')

        response = savory_dispatch(root_resource, method='POST', body='{}')

        self.assertEqual(response.status_code, 405)

    def test_delete_success(self):
        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('DELETE')

        savory_dispatch(root_resource, method='DELETE')
        self.assertTrue(root_resource.delete.called)
        self.assertIsNotNone(root_resource.delete.call_args_list[0].request)

    def test_delete_not_supported(self):
        root_resource = mock_resource(name='root')

        response = savory_dispatch(root_resource, method='DELETE')
        self.assertEqual(response.status_code, 405)

    def test_child_resolution(self):
        child_resource = mock_resource(name='child')
        child_resource.allowed_methods.add('GET')
        child_resource.get = Mock(return_value={})

        root_resource = mock_resource(name='root', child_resource=child_resource)

        savory_dispatch(root_resource, method='GET', resource_path='child')

        self.assertEqual(call_args_sans_context(root_resource.get_child_resource), ['child'])
        self.assertTrue(child_resource.get.called)
        self.assertIsNotNone(child_resource.get.call_args_list[0].request)

    def test_grandchild_resolution(self):
        grand_child_resource = mock_resource(name='grandchild')
        grand_child_resource.allowed_methods.add('GET')
        grand_child_resource.get = Mock(return_value={})

        child_resource = mock_resource(name='child', child_resource=grand_child_resource)

        root_resource = mock_resource(name='root', child_resource=child_resource)

        savory_dispatch(root_resource, method='GET', resource_path='child/grandchild')

        self.assertEqual(call_args_sans_context(root_resource.get_child_resource), ['child'])
        self.assertEqual(call_args_sans_context(child_resource.get_child_resource), ['grandchild'])
        self.assertTrue(grand_child_resource.get.called)
        self.assertIsNotNone(grand_child_resource.get.call_args_list[0].request)

    def test_child_resolution_fail(self):
        root_resource = mock_resource(name='root')

        response = savory_dispatch(root_resource, method='GET', resource_path='child/grandchild')
        self.assertEqual(response.status_code, 404)

    def test_exception_handling(self):
        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('GET')
        root_resource.get.side_effect = Exception('Fail')

        response = savory_dispatch(root_resource, method='GET')

        response_json = json.loads(response.content)
        self.assertIn('error', response_json)
        self.assertEqual(response.status_code, 500)

    def test_validation_handling(self):
        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('PUT')
        root_resource.put.side_effect = validators.ValidationError(
            Mock(),
            {
                'class.field': 'broken',
            }
        )

        response = savory_dispatch(root_resource, method='PUT', body='{}')

        response_json = json.loads(response.content)
        self.assertIn('validation_errors', response_json)
        self.assertEqual(response_json['validation_errors']['class.field'], 'broken')
        self.assertEqual(response.status_code, 400)

    def test_set_header(self):
        """
        Tests the set_header method in the APIContext class
        """

        def get(ctx, params):
            ctx.set_header('foo1', 'bar1')
            return {'foo2': 'bar2'}

        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('GET')
        root_resource.get = get

        response = savory_dispatch(root_resource, method='GET')

        self.assertEqual(response['foo1'], 'bar1')
        self.assertEqual(response.content, '{"foo2": "bar2"}')

    def test_set_expires(self):
        expires = datetime(2005, 7, 14, 12, 30)

        def get(ctx, params):
            ctx.set_expires_header(datetime.utcnow())
            ctx.set_expires_header(expires)
            ctx.set_expires_header(datetime.utcnow())
            return {'foo2': 'bar2'}

        root_resource = mock_resource(name='root')
        root_resource.allowed_methods.add('GET')
        root_resource.get = get

        response = savory_dispatch(root_resource, method='GET')

        self.assertEqual(response['Expires'], expires.isoformat('T'))
        self.assertEqual(response.content, '{"foo2": "bar2"}')


class HashTestCase(unittest.TestCase):
    def test_mutable_parameters(self):
        dct = {'a': 'http://one/two/three/four'}
        ctx = mock_context()
        ctx.formatter = JSONFormatter()

        get_sha1(ctx, dct)
        # Make sure that the dct that we pass in is the same dict that gets returned
        self.assertEqual(dct, {'a': 'http://one/two/three/four'})

    def test_has_dictionary(self):
        dct = {'a': 'b', 'c': 'd'}
        ctx = mock_context()
        ctx.formatter = JSONFormatter()
        self.assertEqual(get_sha1(ctx, dct), '855e751b12bf88bce273d5e1d93a31af9e4945d6')


class DjangoFPramsTest(unittest.TestCase):
    def test_get_list(self):
        get = Mock(name='mock')
        get.getlist.return_value = ['bar', 'baz']
        params = _ParamsImpl(get)

        result = params.get_list('foo')

        self.assertEqual(result, ['bar', 'baz'])
        get.getlist.assert_called_with('foo')



import unittest

from savory_pie.savory_newrelic import get_transaction_name_path


class SavoryPieNewRelicTestCase(unittest.TestCase):

    def _assertMakesInto(self, input, output):
        self.assertEqual(get_transaction_name_path(input), output)

    def _assertDoesNotChange(self, input):
        self._assertMakesInto(input, input)

    def test_does_not_modify_okay_paths(self):
        self._assertDoesNotChange('bars/magic8ball')

        self._assertDoesNotChange('merchandising')

        self._assertDoesNotChange('merchandising/productcontext')

        self._assertDoesNotChange('merchandising/productcontext/')

        self._assertDoesNotChange('merchandising/productcontext/magic99')

    def test_removes_final_id_from_resource_paths(self):
        self._assertMakesInto('bars/foo', 'bars/foo')
        self._assertMakesInto('bars/bar/9', 'bars/bar')
        self._assertMakesInto('bars/19229', 'bars')

        self._assertMakesInto(
            'merchandising/productcontext/493949',
            'merchandising/productcontext'
        )

        self._assertMakesInto(
            'merchandising/productcontext/awesomeness/493949',
            'merchandising/productcontext/awesomeness'
        )



try:
    import ujson as json
except ImportError:
    from warnings import warn
    warn('Using plain JSON instead of uJSON, performance may be degraded.')
    import json
import unittest

from mock import Mock

from savory_pie.resources import APIResource
from savory_pie.django import resources, fields

from savory_pie.tests.django import mock_orm
from savory_pie.tests.django.mock_request import savory_dispatch


class User(mock_orm.Model):
    pass


# Need to mock early, QuerySetResource "rightly" assumes it can call all immediately.
User.objects.all = Mock(return_value=mock_orm.QuerySet(
    User(pk=1, name='Alice', age=31),
    User(pk=2, name='Bob', age=20)
))


class UserResource(resources.ModelResource):
    parent_resource_path = 'users'
    model_class = User

    fields = [
        fields.AttributeField(attribute='name', type=str),
        fields.AttributeField(attribute='age', type=int)
    ]


class UserQuerySetResource(resources.QuerySetResource):
    resource_class = UserResource


api_resource = APIResource()
api_resource.register_class(UserQuerySetResource)


class IntegrationTest(unittest.TestCase):
    def test_get_success(self):
        response = savory_dispatch(api_resource, method='GET', resource_path='users/1')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            json.loads(response.content),
            {u'resourceUri': u'http://localhost/api/users/1',
             u'age': 31,
             u'name': 'Alice'}
        )



import logging
import pprint
import sys
import traceback

from django.db import connection


def getLogger(name=None, stream=None):
    if name is None:
        name = __name__

    logger = logging.getLogger(name)
    formatter = logging.Formatter("[%(funcName)s: %(filename)s:%(lineno)d] %(message)s")
    handler = logging.StreamHandler(stream=stream or sys.stderr)
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    def logger_callable(f, logger=logger):
        assert callable(f)
        if hasattr(f, 'im_func'):
            f = f.im_func
        fc = f.func_code
        info = '{0}:{1} {2}'.format(fc.co_filename, fc.co_firstlineno, fc.co_name)
        logger._log(logging.DEBUG, info, [], {})
    logger.callable = logger_callable

    def logger_alert(obj, marker='*', logger=logger):
        if logger.isEnabledFor(logging.DEBUG):
            bannerEdge = ' '.join(40 * [marker])
            objStr = str(obj)
            logger._log(logging.DEBUG, bannerEdge, [], {})
            logger._log(logging.DEBUG, bannerEdge, [], {})
            logger._log(logging.DEBUG, ((40 - len(objStr) / 2) * ' ') + objStr, [], {})
            logger._log(logging.DEBUG, bannerEdge, [], {})
            logger._log(logging.DEBUG, bannerEdge, [], {})
    logger.alert = logger_alert

    def logger_pprint(obj, logger=logger):
        if logger.isEnabledFor(logging.DEBUG):
            logger._log(logging.DEBUG, '\n' + pprint.pformat(obj), [], {})
    logger.pprint = logger_pprint

    def logger_tb(logger=logger):
        if logger.isEnabledFor(logging.DEBUG):
            message = ''.join(traceback.format_stack())
            logger._log(logging.DEBUG, '\n' + message, [], {})
    logger.tb = logger_tb

    # show database queries
    def logger_before_queries(txt=None, logger=logger):
        if logger.isEnabledFor(logging.DEBUG):
            if txt is not None:
                logger._log(logging.DEBUG, txt, [], {})
            logger._num_queries = len(connection.queries)

    def logger_after_queries(obj=None, logger=logger):
        if logger.isEnabledFor(logging.DEBUG):
            queries = connection.queries[logger._num_queries:]
            if len(queries) > 0:
                logger._log(logging.DEBUG, 'Database hits: {0}'.format(len(queries)), [], {})
                for query in queries:
                    about = 'Database hit, {0} seconds\n'.format(query['time'])
                    logger._log(logging.DEBUG, about + query['sql'], [], {})
            if obj is not None:
                logger._log(logging.DEBUG, '\n' + pprint.pformat(obj), [], {})
            logger._num_queries = len(connection.queries)

    logger.before_queries = logger_before_queries
    logger.after_queries = logger_after_queries
    return logger


logger = getLogger()


class Related(object):
    """
    Helper object that helps build related select-s and prefetch-es.
    Originally created to work around Django silliness - https://code.djangoproject.com/ticket/16855,
    but later extended to help track the related path from the root Model being selected.
    """
    def __init__(self, prefix=None, select=None, prefetch=None, force_prefetch=False):
        self._prefix = prefix
        self._select = select if select is not None else set()
        self._prefetch = prefetch if prefetch is not None else set()
        self._annotate = []
        self._force_prefetch = force_prefetch

    def translate(self, attribute):
        if self._prefix is None:
            return attribute
        else:
            return self._prefix + '__' + attribute

    def select(self, attribute):
        """
        Called to select a related attribute -- this typically translates to a
        select_related call on the final queryset.

        When select is called on a sub-Related created directly or indirectly
        through a sub_prefetch, select-s will automatically be translated into
        prefetch-es.
        """
        # If a select call is made on a Related that was created through sub_prefetch,
        # that call must be converted into prefetch because the relationship to the
        # top element will not have a cardinality of 1.
        if self._force_prefetch:
            return self.prefetch(attribute)

        self._select.add(self.translate(attribute))
        return self

    def prefetch(self, attribute):
        """
        Called to prefetch a related attribute -- this translates into a
        prefetch_related call on the final queryset.
        """
        self._prefetch.add(self.translate(attribute))
        return self

    def sub_select(self, attribute):
        """
        Creates a sub-Related through this relationship.  All calls to select or
        prefetch on the resulting sub-Related will be automatically qualified with
        {attribute}__.

        A sub-select Related acquired through a sub-prefetch Related will continue
        to translates all select-s to prefetch-es.
        """
        return Related(
            prefix=self.translate(attribute),
            select=self._select,
            prefetch=self._prefetch,
            force_prefetch=self._force_prefetch
        )

    def sub_prefetch(self, attribute):
        """
        Creates a sub-Related through this relationship.  All calls to select or
        prefetch on the resulting sub-Related will be automatically qualified with
        {attribute}__.

        Furthermore, all select-s on the sub-related will be translated into
        prefetch-es because they will be read indirectly through a many relationship.
        """
        return Related(
            prefix=self.translate(attribute),
            select=self._select,
            prefetch=self._prefetch,
            force_prefetch=True
        )

    def annotate(self, aggregate, *args, **kwargs):
        """
        Adds an annotation to the current query set. Annotations are always
        added to the end of the query set so all filters will be applied.

        Example usage:
            ``related.aggregate(Count, 'book')``
        """
        self._annotate.append(aggregate(*args, **kwargs))

    def prepare(self, queryset):
        """
        Should be called after all select and prefetch calls have been made to
        applied the accumulated confiugration to a QuerySet.
        """
        if self._select:
            queryset = queryset.select_related(*self._select)

        if self._prefetch:
            queryset = queryset.prefetch_related(*self._prefetch)

        if self._annotate:
            queryset = queryset.annotate(*self._annotate)

        return queryset



import collections
import functools
import importlib
from savory_pie.auth import authorization, authorization_adapter
from savory_pie.resources import EmptyParams
from savory_pie.django.validators import validate, ValidationError
from savory_pie.errors import SavoryPieError


def read_only_noop(func):
    @functools.wraps(func)
    def inner(self, *args, **kwargs):
        if not self._read_only:
            return func(self, *args, **kwargs)
    return inner


class ResourceClassUser(type):
    def __new__(cls, name, bases, d):

        def init_resource_class(self, rclass):
            self._arg_resource_class = rclass
            if isinstance(rclass, str) or isinstance(rclass, unicode):
                self._real_resource_class = None
            else:
                self._real_resource_class = rclass

        def getter(self):
            if self._real_resource_class is None and \
                    self._arg_resource_class is not None:
                rclass = self._arg_resource_class
                n = rclass.rindex('.')
                module = importlib.import_module(rclass[:n])
                self._real_resource_class = getattr(module, rclass[n + 1:])
            return self._real_resource_class

        def setter(self, value):
            self._real_resource_class = value

        deler = None
        d['init_resource_class'] = init_resource_class
        d['_resource_class'] = property(getter, setter, deler, '')
        return type.__new__(cls, name, bases, d)


class Field(object):
    @property
    def name(self):
        name = getattr(self, '_attribute', None) or getattr(self, '_full_attribute', None)
        if not name:
            raise SavoryPieError(u'Unable to determine name for field: {0}'.format(self))
        return name

    def schema(self, ctx, **kwargs):
        schema = kwargs.pop('schema', {})
        if getattr(self, '_type', None):
            return dict({'type': self._type.__name__}.items() + schema.items())
        return schema


class AttributeField(Field):
    """
    Simple Field that translates an object attribute to/from a dict.

        Parameters:

            ``attribute``
                attribute on the Model can be a multi-level expression - like
                related_entity.attribute

            ``type``
                expecting type of value -- int, bool, etc.

            ``published_property``
                optional -- name exposed in the API

            ``read_only``
                optional -- this api will never try and set this value

            ``optional``
                optional -- if missing, will not throw a ValidationError

        .. code-block:: python

            AttributeField('name', type=str)

        .. code-block:: javascript

            {'name': obj.name}

        .. code-block:: python

            AttributeField('other.age', type=int)

        .. code-block:: javascript

           {'age': obj.other.age}
    """
    def __init__(self,
                 attribute,
                 type,
                 published_property=None,
                 use_prefetch=False,
                 read_only=False,
                 validator=None,
                 optional=False,
                 permission=None):
        self._full_attribute = attribute
        self._type = type
        self._published_property = published_property
        self._read_only = read_only
        self._optional = optional
        self.validator = validator or []
        self.permission = permission

    def _compute_property(self, ctx):
        if self._published_property is not None:
            return ctx.formatter.convert_to_public_property(self._published_property)
        else:
            return ctx.formatter.convert_to_public_property(self._bare_attribute)

    @property
    def _bare_attribute(self):
        return self._full_attribute.split('.')[-1]

    @property
    def _attrs(self):
        return self._full_attribute.split('.')

    def _get_object(self, root_obj):
        obj = root_obj
        for attr in self._attrs[:-1]:
            obj = getattr(obj, attr)
            if obj is None:
                return None
        return obj

    def _get(self, obj):
        obj = self._get_object(obj)
        if obj is None:
            return None
        else:
            return getattr(obj, self._bare_attribute)

    def _set(self, obj, value):
        obj = self._get_object(obj)
        # TODO: handle None
        return setattr(obj, self._bare_attribute, value)

    @read_only_noop
    @authorization(authorization_adapter)
    def handle_incoming(self, ctx, source_dict, target_obj):
        attr = self._compute_property(ctx)
        if attr not in source_dict:
            if self._optional:
                return
            raise ValidationError(self, {'missingField': attr,
                                         'target': type(target_obj).__name__})
        with ctx.target(target_obj):
            self._set(
                target_obj,
                self.to_python_value(ctx, source_dict[self._compute_property(ctx)])
            )

    def handle_outgoing(self, ctx, source_obj, target_dict):
        target_dict[self._compute_property(ctx)] = self.to_api_value(
            ctx,
            self._get(source_obj)
        )

    def to_python_value(self, ctx, api_value):
        return ctx.formatter.to_python_value(self._type, api_value)

    def to_api_value(self, ctx, python_value):
        return ctx.formatter.to_api_value(self._type, python_value)

    def validate_resource(self, ctx, key, resource, value):
        error_dict = {}
        if isinstance(self.validator, collections.Iterable):
            for validator in self.validator:
                validator.find_errors(error_dict, ctx, key, resource, self, value)
        else:
            self.validator.find_errors(error_dict, ctx, key, resource, self, value)
        return error_dict


class URIResourceField(Field):
    """
    Field that exposes just the URI of related entity


    Parameters:

        ``attribute``
            name of the relationship between the parent object and the related
            object may only be single level

        ``resource_class``
            a ModelResource -- used to represent the related object needs to be
            fully addressable

        ``published_property``
            optional -- name exposed in the API

        ``read_only``
            optional -- this api will never try and set this value

        .. code-block:: python

            URIResourceField('other', OtherResource)

        .. code-block:: javascript

            {'other': '/api/other/{pk}'}
    """
    __metaclass__ = ResourceClassUser

    def __init__(self,
                 attribute,
                 resource_class,
                 published_property=None,
                 read_only=False,
                 validator=None,
                 permission=None):
        self._attribute = attribute
        self.init_resource_class(resource_class)
        self._published_property = published_property
        self._read_only = read_only
        self.validator = validator or []
        self.permission = permission

    def _compute_property(self, ctx):
        if self._published_property is not None:
            return ctx.formatter.convert_to_public_property(self._published_property)
        else:
            return ctx.formatter.convert_to_public_property(self._attribute)

    @read_only_noop
    @authorization(authorization_adapter)
    def handle_incoming(self, ctx, source_dict, target_obj):
        uri = source_dict[self._compute_property(ctx)]
        if uri is not None:
            resource = ctx.resolve_resource_uri(uri)
            if resource is None:
                raise ValueError('invalid URI {0}: '.format(uri))

            setattr(target_obj, self._attribute, resource.model)
        else:
            setattr(target_obj, self._attribute, None)

    def handle_outgoing(self, ctx, source_obj, target_dict):
        sub_model = getattr(source_obj, self._attribute)
        if sub_model is not None:
            resource = self._resource_class(sub_model)
            target_dict[self._compute_property(ctx)] = ctx.build_resource_uri(resource)
        else:
            target_dict[self._compute_property(ctx)] = None

    def validate_resource(self, ctx, key, resource, source_dict):
        error_dict = {}
        # TODO how do we validate this guy?
        return error_dict


class CompleteURIResourceField(Field):
    """
    Field that exposes just the URI of the complete entity of itself.
    This is useful if a resource is explicitly not including resource_uris, due to recursive inclusion,
    this field can be used, to link to the URI of the full resource version of itself.
    It adds a hard coded 'completeResourceUri' entry to the target dictionary.

    Parameters:

        ``resource_class``
            a ModelResource -- used to represent the related object needs to be
            fully addressable

        .. code-block:: python

            CompleteURIResourceField(OtherResource)

        .. code-block:: javascript

            {'completeResourceUri': '/api/other/{pk}'}
    """
    __metaclass__ = ResourceClassUser

    def __init__(self, resource_class, read_only=False, permission=None):
        self.init_resource_class(resource_class)
        self._read_only = read_only
        self.permission = permission

    @authorization(authorization_adapter)
    def handle_incoming(self, ctx, source_dict, target_obj):
        pass

    def handle_outgoing(self, ctx, source_obj, target_dict):
        resource = self._resource_class(source_obj)
        property_name = ctx.formatter.convert_to_public_property('complete_resource_uri')
        target_dict[property_name] = ctx.build_resource_uri(resource)


class URIListResourceField(Field):
    """
    Field that exposes a list of URIs of related entity, this allows for a many to many relationship.


    Parameters:

        ``attribute``
            name of the relationship between the parent object and the related
            object may only be single level

        ``resource_class``
            a ModelResource -- used to represent the related object needs to be
            fully addressable

        ``published_property``
            optional -- name exposed in the API

        ``read_only``
            optional -- this api will never try and set this value

        .. code-block:: python

            URIListResourceField('others', OtherResource)

        .. code-block:: javascript

            {'others': ['/api/other/{pk_1}', '/api/other/{pk_2}']
    """
    __metaclass__ = ResourceClassUser

    def __init__(self,
                 attribute,
                 resource_class,
                 published_property=None,
                 read_only=False,
                 validator=None,
                 permission=None):
        self._attribute = attribute
        self.init_resource_class(resource_class)
        self._published_property = published_property
        self._read_only = read_only
        self.validator = validator or []
        self.permission = permission

    def _compute_property(self, ctx):
        if self._published_property is not None:
            return ctx.formatter.convert_to_public_property(self._published_property)
        else:
            return ctx.formatter.convert_to_public_property(self._attribute)

    def get_iterable(self, value):
        return value

    @read_only_noop
    @authorization(authorization_adapter)
    def handle_incoming(self, ctx, source_dict, target_obj):
        attribute = getattr(target_obj, self._attribute)

        db_keys = set()
        db_models = {}
        for model in self.get_iterable(attribute):
            resource = self._resource_class(model)
            db_models[resource.key] = model
            db_keys.add(resource.key)

        new_models = []
        request_keys = set()

        for resource_uri in source_dict[self._compute_property(ctx)]:
            resource = ctx.resolve_resource_uri(resource_uri)
            if resource:
                request_keys.add(resource.key)

                if resource.key not in db_keys:
                    new_models.append(resource.model)
            else:
                raise SavoryPieError(u'Unable to resolve resource uri {0}'.format(resource_uri))

        # Delete before add to prevent problems with unique constraints
        models_to_remove = [db_models[key] for key in db_keys - request_keys]
        # If the FK is not nullable the attribute will not have a remove
        if hasattr(attribute, 'remove'):
            attribute.remove(*models_to_remove)
        else:
            for model in models_to_remove:
                model.delete()

        if hasattr(attribute, 'add'):
            attribute.add(*new_models)
        else:
            for obj in new_models:
                through_parameters = {
                    attribute.source_field_name: target_obj,
                    attribute.target_field_name: obj
                }
                attribute.through.objects.create(**through_parameters)

    def handle_outgoing(self, ctx, source_obj, target_dict):
        attrs = self._attribute.split('.')
        attribute = source_obj

        for attr in attrs:
            attribute = getattr(attribute, attr)
            if attribute is None:
                return None

        resource_uris = []
        for model in self.get_iterable(attribute):
            model_resource = self._resource_class(model)
            resource_uris.append(ctx.build_resource_uri(model_resource))
        target_dict[self._compute_property(ctx)] = resource_uris


class SubObjectResourceField(Field):
    """
    Field that embeds a single related resource into the parent object

    Parameters:

        ``attribute``
            name of the relationship between the parent object and the related
            object may only be single level

        ``resource_class``
            a ModelResource -- used to represent the related object

        ``published_property``
            optional -- name exposed in the API

        ``read_only``
            optional -- this api will never try and set this value

        ``validator``
            optional -- a ResourceValidator, or list/tuple of ResourceValidators, to
            validate the data in the related object

        .. code-block:: python

            SubObjectResourceField('other', OtherResource)

        .. code-block:: javascript

            {'other': {'age': 9}}
    """
    __metaclass__ = ResourceClassUser

    def __init__(self,
                 attribute,
                 resource_class,
                 published_property=None,
                 read_only=False,
                 validator=None,
                 permission=None):
        self._attribute = attribute
        self.init_resource_class(resource_class)
        self._published_property = published_property
        self._read_only = read_only
        self.validator = validator or []
        self.permission = permission

    def _compute_property(self, ctx):
        if self._published_property is not None:
            return ctx.formatter.convert_to_public_property(self._published_property)
        else:
            return ctx.formatter.convert_to_public_property(self._attribute)

    def get_subresource(self, ctx, source_dict, target_obj):
        """
        Extention point called by :meth:~`savory_pie.fields.handle_incoming` to
        build a resource class around the target attribute or return None if it
        is not found. Can try looking by resource_uri etc.
        """
        sub_source_dict = source_dict[self._compute_property(ctx)]
        resource = None
        # TODO: clean up later per bug JRUT-4708
        if sub_source_dict is not None and 'resourceUri' in sub_source_dict:
            resource = ctx.resolve_resource_uri(sub_source_dict['resourceUri'])
        else:
            try:
                attribute = getattr(target_obj, self._attribute)
            except AttributeError:
                return None

            resource = self._resource_class(attribute)

        return resource

    def get_submodel(self, ctx, source_object):
        return getattr(source_object, self._attribute, None)

    def pre_save(self, model):
        return True

    @read_only_noop
    @authorization(authorization_adapter)
    def handle_incoming(self, ctx, source_dict, target_obj):
        if not source_dict:
            setattr(target_obj, self._attribute, None)
        else:
            sub_resource = self.get_subresource(ctx, source_dict, target_obj)

            if not sub_resource:  # creating a new resource
                sub_resource = self._resource_class.create_resource()

            sub_source_dict = source_dict[self._compute_property(ctx)]

            # this is to get around django-orm limitations, where in particular
            # if you have a one-to-one field, you can't set it to None since orm doesn't like it
            # so only set the attr to None, if what's coming in is None and what's there is not already None
            if sub_source_dict is None:
                if hasattr(target_obj, self._attribute) \
                   and getattr(target_obj, self._attribute) is not None \
                   and getattr(target_obj, self._attribute).pk:
                    setattr(target_obj, self._attribute, None)
            else:
                # Use the pre_save property, to determine whether we need to set the attribute before or after put
                # in the case of a ReverseSingleRelatedObject (pre_save is False), then we need to set the attribute first
                # before calling put. This is to get around the Django ORM restrictions.
                if not self.pre_save(target_obj):
                    setattr(target_obj, self._attribute, sub_resource.model)

                with ctx.target(target_obj):
                    sub_resource.put(
                        ctx,
                        sub_source_dict,
                        skip_validation=getattr(self, '_skip_validation', False)
                    )

                if self.pre_save(target_obj):
                    setattr(target_obj, self._attribute, sub_resource.model)

    def handle_outgoing(self, ctx, source_obj, target_dict):
        sub_model = self.get_submodel(ctx, source_obj)
        if sub_model is None:
            target_dict[self._compute_property(ctx)] = None
        else:
            target_dict[self._compute_property(ctx)] =\
                self._resource_class(sub_model).get(ctx, EmptyParams())

    def validate_resource(self, ctx, key, resource, source_dict):
        return validate(ctx, key + '.' + self.name, self._resource_class, source_dict)


class IterableField(Field):
    """
    Field that embeds a many relationship into the parent object

    Parameters:

        ``attribute``
            name of the relationship between the parent object and the related
            objects can be a multi-level expression - like related_entity.many_to_many_field

        ``resource_class``
            a ModelResource -- used to represent the related objects

        ``published_property``
            optional name exposed through the API

        ``read_only``
            optional -- this api will never try and set this value

        ``iterable_factory``
            optional -- a callable which is passed the attribute and returns an
            iterable this fields exports

        .. code-block:: python

            RelatedManagerField('others', OtherResource)

        .. code-block:: javascript

            {'others': [{'age': 6}, {'age': 1}]}
    """
    __metaclass__ = ResourceClassUser

    def __init__(self,
                 attribute,
                 resource_class,
                 published_property=None,
                 read_only=False,
                 iterable_factory=None,
                 validator=None,
                 permission=None):
        self._attribute = attribute
        self.init_resource_class(resource_class)
        self._published_property = published_property
        self._read_only = read_only
        self._iterable_factory = iterable_factory
        self.validator = validator or []
        self.permission = permission

    def _compute_property(self, ctx):
        if self._published_property is not None:
            return ctx.formatter.convert_to_public_property(self._published_property)
        else:
            return ctx.formatter.convert_to_public_property(self._attribute)

    def _get_resource(self, ctx, attribute, model_dict):
        resource = None
        if 'resourceUri' in model_dict:
            resource = ctx.resolve_resource_uri(model_dict['resourceUri'])
        elif '_id' in model_dict:  # TODO what if you give an id that is not in the db?
            # TODO get key without the extra db lookup
            model = self._resource_class.get_from_queryset(
                attribute.all(),
                model_dict['_id']
            )
            resource = self._resource_class(model)
        return resource

    def get_iterable(self, value):
        return value

    @property
    def _bare_attribute(self):
        return self._attribute.split('.')[-1]

    @read_only_noop
    @authorization(authorization_adapter)
    def handle_incoming(self, ctx, source_dict, target_obj):
        attribute = getattr(target_obj, self._attribute)

        # We are doing this outside of get_iterable so that subclasses can not
        # remove this override.
        if self._iterable_factory:
            iterable = self._iterable_factory(attribute)
        else:
            iterable = self.get_iterable(attribute)

        db_keys = set()
        db_models = {}
        for model in iterable:
            resource = self._resource_class(model)
            db_models[resource.key] = model
            db_keys.add(resource.key)

        new_models = []
        new_put_data = []
        request_keys = set()
        request_models = {}
        for model_dict in source_dict.get(self._compute_property(ctx), []):
            resource = self._get_resource(ctx, attribute, model_dict)
            if resource:
                request_models[resource.key] = resource.model
                request_keys.add(resource.key)
                # Check to see if the resource has already been saved in the DB
                if resource.key in db_keys:
                    with ctx.target(resource.model):
                        resource.put(ctx, model_dict)
                    # If the resource has been saved to the db and the model is
                    # a RelatedManager that is a through (existence of add attribute)
                    # must add it to the new model since it can create a model based
                    # on just the association.
                    if not hasattr(attribute, 'add'):
                        new_models.append(resource.model)
                else:
                    # if the model is not in the database, must save it
                    new_models.append(resource.model)

            else:
                # if the resource does not exist then this is a new instance
                new_put_data.append(model_dict)

        # Delete before add to prevent problems with unique constraints
        models_to_remove = [db_models[key] for key in db_keys - request_keys]
        # If the FK is not nullable the attribute will not have a remove
        if hasattr(attribute, 'remove'):
            attribute.remove(*models_to_remove)
        else:
            for obj in models_to_remove:
                # ManyRelatedManager
                if hasattr(attribute, 'through'):
                    through_params = {
                        attribute.source_field_name: target_obj,
                        attribute.target_field_name: obj
                    }
                    # only delete intermediary model instance if it already exists
                    for instance in attribute.through.objects.filter(**through_params):
                        instance.delete()
                # RelatedManager
                else:
                    obj.delete()

        # Delay all the new creates untill after the deletes for unique
        # constraints again
        for model_dict in new_put_data:
            model_resource = self._resource_class.create_resource()
            with ctx.target(target_obj):
                model_resource.put(ctx, model_dict, save=True)
            new_models.append(model_resource.model)

        if hasattr(attribute, 'add'):
            attribute.add(*new_models)
        else:
            for obj in new_models:
                through_params = {
                    attribute.source_field_name: target_obj,
                    attribute.target_field_name: obj
                }
                # only create intermediary model instance if it doesn't already exist
                if not attribute.through.objects.filter(**through_params).exists():
                    attribute.through.objects.create(**through_params)

    def handle_outgoing(self, ctx, source_obj, target_dict):
        attrs = self._attribute.split('.')
        attribute = source_obj

        for attr in attrs:
            attribute = getattr(attribute, attr, None)
            if attribute is None:
                return None

        objects = []

        # We are doing this outside of get_iterable so that subclasses can not
        # remove this override.
        if self._iterable_factory:
            iterable = self._iterable_factory(attribute)
        else:
            iterable = self.get_iterable(attribute)

        for model in iterable:
            model_resource = self._resource_class(model)
            model_dict = model_resource.get(ctx, EmptyParams())
            # only add '_id' if there is no 'resourceUri'
            if 'resourceUri' not in model_dict:
                model_dict['_id'] = model_resource.key
            objects.append(model_dict)
        target_dict[self._compute_property(ctx)] = objects

    def validate_resource(self, ctx, key, resource, source_dict_list):
        error_dict = {}
        if self.validator:
            self.validator.find_errors(error_dict, ctx, key, resource, self, source_dict_list)
        return error_dict

    def schema(self, ctx, **kwargs):
        return super(IterableField, self).schema(ctx, **kwargs)
